use ExtUtils::MakeMaker;
#
# See lib/ExtUtils/MakeMaker.pm for details of how to influence
# the contents of the Makefile that is written.
#
# If any modules outside of the core perl distribution are required,
# these should be included as a PREREQ_PM entry in WriteMakefile below, 
# as indicated in the example. This example requires the modules 
# MOD1 and MOD2 to be installed, with minimal versions 1 and 5,
# respectively. If the version number is 0, any version is sufficient.
#
# As well, if you wish to force a minimal perl version to run the
# script, insert a line, for example,
#
#   require 5.004;
#
# below.
require 5.006;  # Need 'autouse'(5.004)
use strict;

use File::Basename;
use File::Spec::Functions qw|:ALL|;
use Config;

my %args = (
            pkg_name  => 'eidetic',
            name      => 'rackview',
            web_user  => 'apache',
            web_group => 'apache',
            httpd_dir => '/etc/httpd/conf',
            cgi_dir   => '/var/www/rackview',
            img_dir   => '/var/www/rackview/images',
	    DESTDIR   => undef 
            );

my @pass_args;
while (my $arg = shift @ARGV) {
    my ($key, $value) = split /=/, $arg;
    if (exists $args{$key}) {
        $args{$key} = $value;
    } else {
        push @pass_args, $arg;
    }
}
@ARGV = @pass_args;

my @scripts = catfile( "scripts", "erackview" );
my @cgis    = catfile( "cgi-bin", "rackview.cgi" );

my %opts = (
	    'INSTALLDIRS'  => 'site',
	    'NAME'	   => $args{name},
	    'VERSION_FROM' => catfile("lib","Eidetic","Racks.pm"),
	    'EXE_FILES'    => [ @scripts ], # scripts to install
	    'PREREQ_PM'    => {
		'File::Spec'     => 0.83, 
		'File::Temp'     => 0.12, 
		'Carp'           => 1.01,
		'GD'             => 1.41,
                'Config::Simple' => 4.1,
                'Eidetic'        => 2,
		},
	    );

if ($ExtUtils::MakeMaker::VERSION >= 5.43) {
    $opts{AUTHOR}   = 'Bryce Harrington <bryce@osdl.org>, Kees Cook <kees@osdl.org>';
    $opts{ABSTRACT} = 'Generate HTML page displaying computer rack layout';
}

# This puts us in the site_perl directory, not dependant on any version
# of perl.
if (defined($Config{'sitelib_stem'}) && $Config{'sitelib_stem'} ne "") {
        print "stem is: $Config{'sitelib_stem'}\n";
        $opts{'INSTALLSITELIB'} = "";
        $opts{'INSTALLSITELIB'} = $args{'DESTDIR'}
                if ($] >= 5.008 ||
                    $ExtUtils::MakeMaker::VERSION =~ /5\.9[1-6]|6\.0[0-5]/);
        $opts{'INSTALLSITELIB'} .= $Config{'sitelib_stem'};
}

WriteMakefile( %opts );

sub install_clause {
    my ($source, $target_dir, $overwrite) = @_;
    $overwrite = 1 unless (defined($overwrite));

    die  "Source '$source' does not exist" unless (-e $source);

    my $text = '';

    $text .= "install :: $source\n";
    $text .= "\t\$(MKPATH) $target_dir\n";
    if ($overwrite) {
        $text .= "\t\$(RSYNC) -Cr $source $target_dir\n";
    } else {
        # Don't over-write
        # (This assumes we're installing a _file_, not a directory tree)
        my $dest_file = catfile( $target_dir, basename( $source ) );
        $text .= "\tif [ -f $dest_file ]; ";
        $text .= "then \$(CP) $source $dest_file.dist; ";
        $text .= "else \$(CP) $source $dest_file; fi\n";
    }
    $text .= "\t\$(CHMOD) -R a+rx $target_dir\n";
    $text .= "\n";
    return $text;
}

sub MY::postamble {
    my $self = shift;
    my $text = '';

    my $etcdir = '';
    my $imgdir = '';
    my $cgidir = '';

    $text .= "RSYNC = rsync\n";
    $text .= "CHOWN = chown\n\n";

    # Determine location of etc conf files
    if ($self->{PREFIX} !~ m|^/usr| &&
        $self->{PREFIX} !~ m|^/usr/local| &&
        !defined($args{DESTDIR})) {
	die <<'EOM';
If you are installing this anywhere other than "/usr" or "/usr/local", you
must specify the "DESTDIR" option on the "make" command line, otherwise
this script cannot make sense of the PREFIX setting.
EOM
    }

    my $destdir = $args{DESTDIR} || rootdir();
    my $prefix  = $self->{PREFIX};

    if ($prefix !~ /^$destdir/) {
        $prefix = "$destdir$prefix";
    }

    $etcdir = catdir( $destdir, 'etc', $args{pkg_name} );
    $imgdir = catdir( $destdir, rootdir(), $args{img_dir} );
    $cgidir = catdir( $destdir, rootdir(), $args{cgi_dir} );

    my $etcfiles = catfile( "etc", "$args{name}.conf" );
    $text .= install_clause($etcfiles, $etcdir, 0);

    # Install the example data file
    my $exampledata = catfile( "example", "example.dat" );
    $text .= install_clause($exampledata, $etcdir, 0);

    # Install cgi-bin files
    foreach my $file ( @cgis ) {
      $text .= install_clause($file, $cgidir);
    }

    # Create images directory
    $text .= "\n\ninstall :: \n";
    $text .= "\t\$(MKPATH) $imgdir\n";
    $text .= "\t\$(CHMOD) a+rx $imgdir\n";
    $text .= "\t\$(CHOWN) -R $args{web_user}:$args{web_group} $imgdir\n";
    $text .= "\n";

    # Install images
    my $imagefiles = catfile( "images", "blank.gif" );
    $text .= install_clause($imagefiles, $imgdir);

    return $text;
}
