#!/usr/bin/perl -w

# This is a cgi program that produces HTML for displaying racks.

use strict;
use Eidetic::Database;
use Eidetic::Racks;
use Config::Simple;
use File::Spec::Functions;
use CGI;

my $NAME="rackview";
my $CONFDIR="/etc/eidetic";

#========================================================================
# CGI parameter handling
#------------------------------------------------------------------------
my $cgi = new CGI;

# Content-type header
print $cgi->header();

my $frame    = $cgi->param('frame');
my @racks;
my $title;
if (!defined($frame)) {
    # Extract rack id #'s and ensure there are no strange characters
    my $rack_ids = $cgi->param('rack_ids');

    $rack_ids = 0 unless ($rack_ids && $rack_ids =~ /^[\w\,]+$/);
    @racks = split /,/, $rack_ids;

    $title="Rackview : Racks ".join(" ",@racks);
}
else {
    $title="Rackview : Frame $frame";
}


#========================================================================
# Config file loading
#------------------------------------------------------------------------
my %appconf;
my $confname=catfile($CONFDIR,"$NAME.conf");
if (!Config::Simple->import_from($confname,\%appconf)) {
    die "$confname: ".Config::Simple->error()."\n";
}

# Set up display defaults
my %args;

# Set up any variables that we won't allow the web form to override
foreach my $arg (qw(
                    header_include footer_include javascript_images
                    )) {
    $args{$arg}=$appconf{$arg} if (defined($appconf{$arg}));
}

# Set up any variables that we allow the web form to override
foreach my $arg (qw(
                    heat rack_heat frame_heat warn alpha umarks
                    printable
                    )) {
    $args{$arg}=$cgi->param($arg);
    #print "$arg from cgi: $args{$arg}<BR>\n";
    next if (defined($args{$arg}));
    $args{$arg}=$appconf{$arg};
    #print "$arg from app: $args{$arg}<BR>\n";
    next if (defined($args{$arg}));
    $args{$arg} = undef;
    #print "$arg default: $args{$arg}<BR>\n";
}

#print "<h2>$title</h2>\n";

#========================================================================
# Process rack data
#------------------------------------------------------------------------
my $db;
my $dat_file;

my $racks;
foreach my $order_option (split(/\,/,$appconf{search_order})) {

    # Attempt to load from file, if it exists
    if ($order_option eq 'dat') {
        $racks = new Eidetic::Racks(
				    filename       => $appconf{dat_file},
				    frame          => $frame,
				    rack_ids       => \@racks,
				    img_dir        => $appconf{img_dir},
				    img_url        => $appconf{img_url},
				    host_viewer    => $appconf{host_viewer},
				    overwrite_pngs => 1
				    );
        last if $racks;
    } elsif ($order_option eq 'db') {
        my $db = Eidetic::Database->new(
					brand      => $appconf{db_brand},
					name       => $appconf{db_name},
					host       => $appconf{db_host},
					user       => $appconf{db_user},
					pass       => $appconf{db_pass}
					);

        $racks = new Eidetic::Racks(
				    database       => $db,
				    frame          => $frame,
				    rack_ids       => \@racks,
				    img_dir        => $appconf{img_dir},
				    img_url        => $appconf{img_url},
				    host_viewer    => $appconf{host_viewer},
				    overwrite_pngs => 1
				    );
        last if $racks;
    } else {
        carp("Unknown option '$order_option' specified\n");
    }
}

if (!$args{'printable'} &&
    defined($args{'header_include'}) && 
    -r $args{'header_include'}) {
    open(FILE,"<$args{'header_include'}") || die "$args{'header_include'}: $!\n";
    grep(print,<FILE>);
    close(FILE);
} elsif (defined($args{'javascript_images'})) {
    print $cgi->start_html(-title=> "Rackview " . join(" ",@racks),
                           -script=> javascript_images()
                           );
} else {
    print $cgi->start_html($title);

}

  
#========================================================================
# Display page
#------------------------------------------------------------------------
if ($racks) {
    print $racks->toHtml(%args);
    if (!$args{'printable'}) {
        printf("<a href=\"%s&printable=1&heat=0&alpha=1&warn=0\">Printable version</a><br>\n",
               $cgi->self_url);
    }
} else {
    print "ERROR:  Could not get find rack data source as file or database\n";
}

if (!$args{'printable'} &&
    defined($args{'footer_include'}) &&
    -r $args{'footer_include'}) {
    open(FILE,"<$args{'footer_include'}") || die "$args{'footer_include'}: $!\n";
    grep(print,<FILE>);
    close(FILE);
} else {
    print $cgi->end_html();
}


#========================================================================
# Create javascript - contributed by Marcus Vogt (mgvogt@bigpond.com.au)
# 
# rackview.cgi is modified to allow the javascript image flipping -
# allows users to actually see the item.  I've hardcoded it to be jpg's
# for the moment.  It would be nice to have a seperate data structure
# for models, as you could then say I have n of model Y and these are
# the power/cooling/image/etc.
#
# Racks.pm is modified to allow for more of the javascript stuff and
# also to do a search and replace on rackview.conf to replace the host
# #name as well.  This allows for the integration into nagios - works
# just #fine here.
#------------------------------------------------------------------------
sub javascript_images {
    my $JAVASCRIPT = "";

    # Must find out the distinct models.

    my %rids;
    my %models;

    my @hosts = values %{$racks->{_hosts}};

    my @idracks = split /,/, $cgi->param('rack_ids');

    foreach my $rid (@idracks) {
        $rids{"$rid"} = 1;
    }

    foreach my $host (@hosts) {
        my $num = "$host->{Rack_number}";
        if (!defined($rids{$num})) {
            next;
        }

        $models{$host->{Brand_model_name}} = $host->{Size_in_rack_u} * $racks->{_u_pixels};

        $JAVASCRIPT .= "$host->{descriptor}_image_MouseOut = new Image($racks->{_rack_width}, " . $models{$host->{Brand_model_name}} . ");\n";+  $JAVASCRIPT .= "$host->{descriptor}_image_MouseOut.src = \"" . $racks->{_img_url} . $host->{descriptor} . ".png" . "\";\n";

        $JAVASCRIPT .= "function $host->{descriptor}_MouseOut() { document.$host->{descriptor}.src = $host->{descriptor}_image_MouseOut.src; return true; }\n";
        $JAVASCRIPT .= "function $host->{descriptor}_MouseOver() { document.$host->{descriptor}.src = $host->{Brand_model_name}_image_MouseOver.src; return true; }\n";
    }

    my @models = keys %models;

    # Now that we have, lets create hovers for them.

    foreach my $model (@models) {
        $JAVASCRIPT .= $model . "_image_MouseOver = new Image($racks->{_rack_width}, " . $models{$model} . ");\n";
        $JAVASCRIPT .= $model . "_image_MouseOver.src = \"" . $racks->{_img_url} . $model . ".jpg" . "\";\n";
    }
    return $JAVASCRIPT;
}

__END__

=head1 NAME

rackview.cgi - Create an HTML view of a set of racks.  Runnable as a CGI
program.

=head1 SYNOPSIS

rackview.cgi?rack_ids=RACK_ID[,RACK_ID]

=head1 OPTIONS

rack_ids - comma separated list of ID's to display.

=head1 DESCRIPTION

B<rackview.cgi> is a CGI wrapper to the Eidetic::Racks perl module
that enables generation of HTML and images for viewing a set of computer
racks.  It's intended to be used with the Eidetic Lab Hardware
Management System for showing the layout of a computer lab.

=head1 PREREQUISITES

This script requires the C<GD> module.  It requires Eidetic::DBObject.
It also requires File::Spec, which typically come with the perl
distribution.

=head1 COREQUISITES

None

=head1 SCRIPT CATEGORIES

CPAN/Administrative

=head1 BUGS

None known.

=head1 SEE ALSO

L<Eidetic::Racks(3)>

=head1 AUTHOR

Bryce Harrington E<lt>brycehar@bryceharrington.comE<gt>.

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2003 by Bryce Harrington

This program is free software; you may redistribute it and/or modify
it under the same terms ans Perl itself.

=head1 REVISION

Revision: $Revision: 1.26 $

=cut

# vi:set ai ts=4 sw=4 expandtab:
